// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

part of omnibus;

/// Internal class for awaiting a dynamic set of async processes.
class _DeepFutureAwaiter {
  // Futures of processes that are currently running.
  Set<Future> runningProcesses = new Set<Future>();

  // Completed when there is silence again.
  Completer _onSilence = new Completer();

  // Add a new process. The returned Future will complete when there is complete
  // silence in the running processes stack.
  Future addProcess(Future process) {
    runningProcesses.add(process);
    process.then((_) {
      // Remove self from the processes stack.
      runningProcesses.remove(process);

      // If there are no running processes left the _onSilence future can be
      // completed and replaced.
      if (runningProcesses.isEmpty) {
        _onSilence.complete();
        _onSilence = new Completer();
      }
    });
    return _onSilence.future;
  }
}

/// Development version of Omnibus that can await nested events.
class OmnibusDevel extends Omnibus {
  /// Helper for watching a dynamic set of async processes.
  _DeepFutureAwaiter _awaiter = new _DeepFutureAwaiter();

  /// Await nested events as well (events that are published by handlers in
  /// response to this message).
  Future publishEvent(OmnibusEvent event) {
    return _awaiter.addProcess(super.publishEvent(event));
  }

  /// Await nested events as well (events that are published by handlers in
  /// response to this message).
  Future<OmnibusResponses> publishRequest(OmnibusRequest request) {
    Completer<OmnibusResponses> completer = new Completer<OmnibusResponses>();
    Future<OmnibusResponses> process = super.publishRequest(request);
    process.then((OmnibusResponses responses) {
      _awaiter.addProcess(responses.onComplete()).then((_) {
        completer.complete(responses);
      });
    });

    return completer.future;
  }
}
