// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

library omnibus.test.devel_test;

import 'dart:async';

import 'package:test/test.dart';
import 'package:omnibus/omnibus.dart';
import 'package:omnibus/patterns.dart';

import 'messages_lib.dart' as v0;

var abcCount = 0;

void main() {
  test('Nested events with the normal Omnibus', () async {
    abcCount = 0;
    var bus = new Omnibus();

    // 1st and 2nd level handlers
    bus.bind(new v0.DataHandler(firstLvlHandler, new BusState(bus)));
    bus.bind(new v0.AbcHandler(secondLvlHandler, new EmptyState()));

    // Request handlers
    bus.bind(new v0.FakeRequestHandler(fakeRequestHandler, new BusState(bus)));

    // 1st level event
    await bus.event(new v0.Data({}));

    // Request
    await bus.request(new v0.FakeRequest({}, {}));

    // With the normal omnibus abcCount should still be 0.
    expect(abcCount, equals(0));
  });

  test('Nested events with the Development Omnibus', () async {
    abcCount = 0;
    var bus = new OmnibusDevel();

    // 1st and 2nd level handlers
    bus.bind(new v0.DataHandler(firstLvlHandler, new BusState(bus)));
    bus.bind(new v0.AbcHandler(secondLvlHandler, new EmptyState()));

    // Request handlers
    bus.bind(new v0.FakeRequestHandler(fakeRequestHandler, new BusState(bus)));

    // 1st level event
    await bus.event(new v0.Data({}));

    // Now abcCount should be 1.
    expect(abcCount, equals(1));

    // Request
    await bus.request(new v0.FakeRequest({}, {}));

    // Now abcCount should be 2.
    expect(abcCount, equals(2));
  });
}

Future<v0.FakeResponse> fakeRequestHandler(
    v0.FakeRequest req, BusState state) async {
  state.bus.event(new v0.Abc());
  return new v0.FakeResponse();
}

Future<Null> firstLvlHandler(v0.Data data, BusState state) async {
  state.bus.event(new v0.Abc());
}

Future<Null> secondLvlHandler(v0.Abc abc, EmptyState state) async {
  abcCount++;
}
